from django.test import TestCase
from django.test import SimpleTestCase
from django.urls import reverse
from django.forms import Form
from phonenumber_field.modelfields import PhoneNumberField
import logging

from django_phonedir.models import Department, FaxNumber, Contact

logger = logging.getLogger(__name__)

try:
    # Django >= 1.7
    from django.test import override_settings
except ImportError as err:
    # Django <= 1.6
    logger.error(err)
    raise RuntimeError(
        "Could not import django.test. No support for Django versions 1.6 or less."
    )

try:
    # Django >= 1.7
    from django.urls import reverse
except ImportError as err:
    # Django <= 1.6
    logger.error(err)
    raise RuntimeError(
        "Could not import django.urls. No support for Django versions 1.6 or less."
    )


# Define test form.
class PhoneFormTest(Form):
    number = PhoneNumberField()


class PhoneDirTests(TestCase):
    def test_models(self):
        # Testing Department model.
        dept = Department(
            name="Information Technology", short_name="IT", supervisor=None
        )
        phoneFormTest = PhoneFormTest({"number": "+1 555 555 5555"})
        self.assertTrue(phoneFormTest.is_valid())

        # Testing FaxNumber model.
        faxNumberTest = FaxNumber(
            department=dept.pk,
            description="Test Fax",
            phone=phoneFormTest.fields.get("number"),
            location="Test Location",
        )
        self.assertEqual(str(faxNumberTest), "+1 555 555 5555 Information Technology")

        # Testing Contact model.
        contactTest = Contact(
            department=dept.pk,
            first_name="Testfirstname",
            last_name="Testlastname",
            title="CEO",
            extension="21111",
            location="CEO Office",
            phone="555-555-1111",
        )
        self.assertEqual(str(contactTest), "Testfirstname Testlastname (Ext: 21111)")

    def test_url_exists_at_correct_location(self):
        response = self.client.get("departments")
        self.assertEqual(response.status_code, 200)
        response = self.client.get("search")
        self.assertEqual(response.status_code, 200)

    def test_url_available_by_name(self):
        response = self.client.get(reverse("home"))
        self.assertEqual(response.status_code, 200)

    def test_template_name_correct(self):
        response = self.client.get(reverse("home"))
        self.assertTemplateUsed(
            response, "django_phonedir/templates/django_phonedir/home.html"
        )

    def test_template_content(self):
        response = self.client.get(reverse("departments"))
        self.assertContains(response, "<h1>Homepage</h1>")
        self.assertNotContains(response, "Not on the page")



from django.contrib import admin

from django_phonedir.models import Contact, Department, FaxNumber


@admin.register(Contact)
class ContactAdmin(admin.ModelAdmin):
    list_filter = ["department"]


class ContactInline(admin.StackedInline):
    model = Contact
    # fields = [("first_name", "last_name", "title", "extension"), ("location", "phone")]
    fieldsets = [
        (None, {"fields": ["first_name", "last_name", "extension", "phone"]}),
        (
            "More options",
            {"classes": ["wide", "collapse"], "fields": ["title", "location"]},
        ),
    ]
    extra = 0  # Provides one empty slot for a new contact


@admin.register(FaxNumber)
class FaxNumberAdmin(admin.ModelAdmin):
    list_filter = ["department"]


class FaxNumberInline(admin.TabularInline):
    model = FaxNumber
    extra = 0  # Provides one empty slot for a new contact


@admin.register(Department)
class DepartmentAdmin(admin.ModelAdmin):
    list_display = ("name", "supervisor")
    inlines = [ContactInline, FaxNumberInline]

    def has_module_permission(self, request):
        # Only allows users with is_staff = True to see this in the admin
        return request.user.is_staff

    def has_permission(self, request, obj=None):
        return request.user.is_staff



from django.conf import settings
from django.db import models
from django.urls import reverse
from phonenumber_field.modelfields import PhoneNumberField

User = settings.AUTH_USER_MODEL


class Department(models.Model):
    name = models.CharField(
        max_length=64, unique=True, help_text="Name of the department."
    )
    short_name = models.CharField(
        max_length=8, unique=True, help_text="Short name that is used in the URL."
    )

    # Points to a User (the supervisor)
    supervisor = models.ForeignKey(
        User,
        on_delete=models.SET_NULL,
        null=True,
        related_name="supervised_departments",
        help_text="Points to a user, typically the supervisor, or any user that will manage the contacts associate with this department.",
    )

    def __str__(self):
        return self.name

    def get_absolute_url(self):
        return reverse("department_detail", kwargs={"short_name": self.short_name})


class FaxNumber(models.Model):
    department = models.ForeignKey(
        Department, on_delete=models.CASCADE, related_name="faxnumbers"
    )
    description = models.CharField(max_length=64)
    phone = PhoneNumberField()
    location = models.CharField(max_length=64)

    def __str__(self):
        return "{phone} {department}".format(
            phone=self.phone, department=self.department
        )


class Contact(models.Model):
    department = models.ForeignKey(
        Department, on_delete=models.CASCADE, related_name="contacts"
    )
    first_name = models.CharField(max_length=64)
    last_name = models.CharField(max_length=64)
    title = models.CharField(max_length=64)
    extension = models.IntegerField()
    location = models.CharField(max_length=64)
    phone = PhoneNumberField(blank=True)

    def __str__(self):
        return "{first_name} {last_name} (Ext: {extension})".format(
            first_name=self.first_name,
            last_name=self.last_name,
            extension=self.extension,
        )



from django.db.models import Q, query
from django.views.generic import DetailView, ListView

from django_phonedir.models import Contact, Department


class DepartmentListView(ListView):
    model = Department
    template_name = "django_phonedir/departments_listing.html"
    context_object_name = "departments"
    ordering = "name"

    # def get_queryset(self, *args, **kwargs):
    # We prefetch 'contacts' to avoid the N+1 query problem
    # return Department.objects.prefetch_related("contacts").all()


class DepartmentDetailView(DetailView):
    model = Department
    template_name = "django_phonedir/department_detail.html"
    slug_field = "short_name"
    slug_url_kwarg = "short_name"


class SearchResultsView(ListView):
    model = Contact
    template_name = "django_phonedir/search_contact_results.html"
    context_object_name = "contacts"

    def get_queryset(self) -> query.QuerySet[Contact]:
        query_request = self.request.GET.get("q")
        contact_list = Contact.objects.filter(
            Q(last_name__icontains=query_request)
            | Q(first_name__icontains=query_request)
        ).order_by("last_name")
        return contact_list
